using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace ColorView
{
   /// <summary>
   /// Custom control to display a matrix of
   /// Colors
   /// </summary>
   public partial class ColorSet : UserControl
   {
      const int COLS = 13;
      const int ROWS = 13;
      const int SZ = 23;
      const int SPACE = 1;

      private List<Color> _colors;
      private ToolTip _toolTip;
      public event EventHandler<ColorSelectedEventArgs> ColorSelected;

      public ColorSet()
      {
         // Initialize the existing component
         // And see where this leads us
         InitializeComponent();
         Name = "Something about mary";
         this.DoubleBuffered = true;
         _toolTip = new ToolTip();
         _toolTip.InitialDelay = 0;
         _toolTip.ShowAlways = false;
         MinimumSize = new Size((COLS * (SZ + SPACE))-1, (ROWS * (SZ + SPACE))-1);
         MaximumSize = MinimumSize;
      }

      public void SortColors(ColorSort sort)
      {
         if ( _colors != null )
         {
            switch ( sort )
            {
            case ColorSort.ByRgb:
               _colors.Sort(SortByRgb);
               break;
            case ColorSort.ByName:
               _colors.Sort(SortByName);
               break;
            case ColorSort.ByBrightness:
               _colors.Sort(SortByBrightness);
               break;
            }
            Invalidate();
         }
      }

      private int SortByRgb(Color c1, Color c2)
      {
         return -(c1.ToArgb().CompareTo(c2.ToArgb()));
      }

      private int SortByBrightness(Color c1, Color c2)
      {
         return -c1.GetBrightness().CompareTo(c2.GetBrightness());
      }

      private int SortByName(Color c1, Color c2)
      {
         return c1.Name.CompareTo(c2.Name);
      }

      protected override void OnPaint(PaintEventArgs e)
      {
         base.OnPaint(e);
         List<Color> colors = GetColors();

         Graphics g = e.Graphics;
         Rectangle rect = this.ClientRectangle;
         g.FillRectangle(SystemBrushes.Window, rect);

         int x = 0, y = 0;
         for ( int i = 0; i < ROWS; i++ )
         {
            x = 0;
            for ( int j = 0; j < COLS; j++ )
            {
               int index = (i * COLS) + j;
               Color color = index < colors.Count ? 
                  colors[index] : Color.White;

               using ( Brush brush = new SolidBrush(color) )
               {
                  g.FillRectangle(brush, x, y, SZ, SZ);
               }
               x += SZ + SPACE;
            }
            y += SZ + SPACE;
         }
      }

      protected override void OnMouseMove(MouseEventArgs e)
      {
         base.OnMouseMove(e);
         int row = e.X / (SZ+SPACE);
         int col = e.Y / (SZ+SPACE);
         if ( row >= ROWS || col >= COLS )
         {
            _toolTip.Hide(this);
            return;
         }

         Color color = GetColorAt(row, col);

         string text = string.Format("{0} ({1},{2},{3})", 
            color.Name, color.R, color.G, color.B);
         _toolTip.Show(text, this, e.X + 20, e.Y);
      }

      protected override void OnMouseLeave(EventArgs e)
      {
         base.OnMouseLeave(e);
         _toolTip.Hide(this);
      }

      protected override void OnMouseClick(MouseEventArgs e)
      {
         base.OnMouseClick(e);
         int row = e.X / (SZ + SPACE);
         int col = e.Y / (SZ + SPACE);
         Color color = GetColorAt(row, col);

         if ( ColorSelected != null )
         {
            ColorSelectedEventArgs args =
               new ColorSelectedEventArgs(color, e.Button == MouseButtons.Right);
            ColorSelected(this, args);
         }
      }

      private Color GetColorAt(int row, int col)
      {
         List<Color> colors = GetColors();
         int index = (col * COLS) + row;
         return index < colors.Count ?
            colors[index] : Color.White;
      }


      private List<Color> GetColors()
      {
         if ( _colors == null )
         {
            Type type = typeof(Color);
            PropertyInfo[] properties =
               type.GetProperties(BindingFlags.Static | BindingFlags.Public);

            _colors = new List<Color>();
            for ( int i = 0; i < properties.Length; i++ )
            {
               _colors.Add((Color)properties[i].GetValue(null, null));
            }
            for ( int i = 0; i < 256; i += 16 )
            {
               Color color = Color.FromArgb(i, i, i);
               if ( !_colors.Contains(color) )
                  _colors.Add(color);
            }
            SortColors(ColorSort.ByRgb);
         }

         return _colors;
      }
   }

   public enum ColorSort
   {
      ByName,
      ByRgb,
      ByBrightness
   }


   public class ColorSelectedEventArgs : EventArgs
   {
      private Color _color;
      private bool _isBackground;

      public bool IsBackground
      {
         get { return _isBackground; }
      }

      public Color Color
      {
         get { return _color; }
      }

      public ColorSelectedEventArgs(Color color, bool isBackground)
      {
         _color = color;
         _isBackground = isBackground;
      }
   }
}
